@@ -35,8 +35,7 @@
 #include "eel-types.h"
 #include "eel-type-builtins.h"
 #include <gtk/gtkprivate.h>
-#include <gtk/gtkselection.h>
-#include <gtk/gtksignal.h>
+#include <gtk/gtk.h>
 #include <eel/eel-canvas.h>
 #include <eel/eel-canvas-util.h>
 #include <gio/gio.h>
@@ -49,8 +48,6 @@ static void       eel_background_class_init                (gpointer       klass
 static void       eel_background_init                      (gpointer       object,
 							    gpointer       klass);
 static void       eel_background_finalize                  (GObject       *object);
-static void       eel_background_reload_image              (EelBackground *background);
-static void       eel_background_ensure_image_nonswapped   (EelBackground *background);
 static GdkPixmap *eel_background_get_pixmap_and_color      (EelBackground *background,
 							    GdkWindow     *window,
 							    GdkColor      *color,
@@ -76,10 +73,7 @@ static guint signals[LAST_SIGNAL];
 struct EelBackgroundDetails {
 	char *color;
 	
-	char *image_uri;
-	time_t image_mtime;
 	GnomeBG *bg;
-	EelBackgroundImagePlacement image_placement;
 
 	/* Realized data: */
 	gboolean background_changes_with_size;
@@ -188,7 +182,6 @@ eel_background_finalize (GObject *object)
 	background = EEL_BACKGROUND (object);
 
 	g_free (background->details->color);
-	g_free (background->details->image_uri);
 	eel_background_remove_current_image (background);
 
 	if (background->details->background_pixmap != NULL) {
@@ -201,12 +194,50 @@ eel_background_finalize (GObject *object)
 	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
+static EelBackgroundImagePlacement
+placement_gnome_to_eel (GnomeBGPlacement p)
+{
+	switch (p) {
+	case GNOME_BG_PLACEMENT_CENTERED:
+		return EEL_BACKGROUND_CENTERED;
+	case GNOME_BG_PLACEMENT_FILL_SCREEN:
+		return EEL_BACKGROUND_SCALED;
+	case GNOME_BG_PLACEMENT_SCALED:
+		return EEL_BACKGROUND_SCALED_ASPECT;
+	case GNOME_BG_PLACEMENT_ZOOMED:
+		return EEL_BACKGROUND_ZOOM;
+	case GNOME_BG_PLACEMENT_TILED:
+		return EEL_BACKGROUND_TILED;
+	}
+
+	return EEL_BACKGROUND_TILED;
+}
+
+static GnomeBGPlacement
+placement_eel_to_gnome (EelBackgroundImagePlacement p)
+{
+	switch (p) {
+	case EEL_BACKGROUND_CENTERED:
+		return GNOME_BG_PLACEMENT_CENTERED;
+	case EEL_BACKGROUND_SCALED:
+		return GNOME_BG_PLACEMENT_FILL_SCREEN;
+	case EEL_BACKGROUND_SCALED_ASPECT:
+		return GNOME_BG_PLACEMENT_SCALED;
+	case EEL_BACKGROUND_ZOOM:
+		return GNOME_BG_PLACEMENT_ZOOMED;
+	case EEL_BACKGROUND_TILED:
+		return GNOME_BG_PLACEMENT_TILED;
+	}
+
+	return GNOME_BG_PLACEMENT_TILED;
+}
+
 EelBackgroundImagePlacement
 eel_background_get_image_placement (EelBackground *background)
 {
 	g_return_val_if_fail (EEL_IS_BACKGROUND (background), EEL_BACKGROUND_TILED);
 
-	return background->details->image_placement;
+	return placement_gnome_to_eel (gnome_bg_get_placement (background->details->bg));
 }
 
 void
@@ -215,11 +246,8 @@ eel_background_set_image_placement (EelBackground              *background,
 {
 	g_return_if_fail (EEL_IS_BACKGROUND (background));
 
-	if (new_placement != background->details->image_placement) {
-		background->details->image_placement = new_placement;
-
-		set_image_properties (background);
-	}
+	gnome_bg_set_placement (background->details->bg,
+				placement_eel_to_gnome (new_placement));
 }
 
 EelBackground *
@@ -228,17 +256,6 @@ eel_background_new (void)
 	return EEL_BACKGROUND (g_object_new (EEL_TYPE_BACKGROUND, NULL));
 }
 
-static void
-eel_background_ensure_image_nonswapped (EelBackground *background)
-{
-	if (!background->details->is_desktop) {
-		return;
-	}
-	if (background->details->image_uri != NULL && background->details->bg == NULL) {
-		eel_background_reload_image (background);
-	}	
-}
-
 static void
 eel_background_unrealize (EelBackground *background)
 {
@@ -285,10 +302,6 @@ eel_background_ensure_realized (EelBackground *background, GdkWindow *window)
 	
 	drawable_get_adjusted_size (background, window, &entire_width, &entire_height);
 	
-	/* Try to parse the color spec.  If we fail, default to the style's color */
-	
-        eel_background_ensure_image_nonswapped (background);
-
 	/* Set the default color */
 	
 	/* Get the widget to which the window belongs and its style as well */
@@ -432,8 +445,6 @@ eel_background_expose (GtkWidget                   *widget,
 static void
 set_image_properties (EelBackground *background)
 {
-	GnomeBGPlacement placement;
-
 	if (!background->details->color) {
 		gnome_bg_set_color (background->details->bg, GNOME_BG_COLOR_SOLID,
 				 &background->details->default_color, NULL);
@@ -462,29 +473,6 @@ set_image_properties (EelBackground *background)
 			gnome_bg_set_color (background->details->bg, GNOME_BG_COLOR_V_GRADIENT, &c1, &c2);
 
 	}
-
-	switch (background->details->image_placement) {
-	case EEL_BACKGROUND_CENTERED:
-		placement = GNOME_BG_PLACEMENT_CENTERED;
-		break;
-	case EEL_BACKGROUND_SCALED:
-		placement = GNOME_BG_PLACEMENT_FILL_SCREEN;
-		break;
-	case EEL_BACKGROUND_SCALED_ASPECT:
-		placement = GNOME_BG_PLACEMENT_SCALED;
-		break;
-	case EEL_BACKGROUND_ZOOM:
-		placement = GNOME_BG_PLACEMENT_ZOOMED;
-		break;
-	case EEL_BACKGROUND_TILED:
-		placement = GNOME_BG_PLACEMENT_TILED;
-		break;
-	default:
-		placement = GNOME_BG_PLACEMENT_TILED;
-		break;
-	}
-	
-	gnome_bg_set_placement (background->details->bg, placement);
 }
 
 char *
@@ -498,9 +486,13 @@ eel_background_get_color (EelBackground *background)
 char *
 eel_background_get_image_uri (EelBackground *background)
 {
+	const char *filename;
+	
 	g_return_val_if_fail (EEL_IS_BACKGROUND (background), NULL);
 
-	return g_strdup (background->details->image_uri);
+	filename = gnome_bg_get_filename (background->details->bg);
+	
+	return g_filename_to_uri (filename, NULL, NULL);
 }
 
 /* Use style->base as the default color instead of bg */
@@ -523,70 +515,39 @@ eel_background_set_color (EelBackground *background,
 	}
 }
 
-static void
-eel_background_reload_image (EelBackground *background)
-{
-	gnome_bg_set_uri (background->details->bg, background->details->image_uri);
-}
-
 static gboolean
 eel_background_set_image_uri_helper (EelBackground *background,
 				     const char *image_uri,
-				     gboolean emit_setting_change,
-				     gboolean emit_appearance_change)
+				     gboolean emit_signal)
 {
-        time_t mtime;
-	GFile *f;
-	GFileInfo *info;
+	char *filename;
 
-	g_return_val_if_fail (EEL_IS_BACKGROUND (background), FALSE);
-
-	mtime = background->details->image_mtime;
 	if (image_uri != NULL) {
-		mtime = 0;
-		f = g_file_new_for_uri (image_uri);
-		info = g_file_query_info (f, G_FILE_ATTRIBUTE_TIME_MODIFIED, 0, NULL, NULL);
-		g_object_unref (f);
-		if (info != NULL) {
-			mtime = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
-			g_object_unref (info);
-		}
+		filename = g_filename_from_uri (image_uri, NULL, NULL);
 	}
-	if (eel_strcmp (background->details->image_uri, image_uri) == 0 &&
-	    mtime == background->details->image_mtime) {
-		return FALSE;
+	else {
+		filename = NULL;
 	}
-
-	g_free (background->details->image_uri);
-	background->details->image_uri = g_strdup (image_uri);
-        background->details->image_mtime = mtime;
-
-	/* We do not get rid of the current image here. This gets done after the new
-	 * image loads - in eel_background_load_image_callback. This way the
-	 * current image can be used if an update is needed before the load completes.
-	 */
 	
-	eel_background_reload_image (background);
-	
-	if (emit_setting_change) {
+	gnome_bg_set_filename (background->details->bg, filename);
+
+	if (emit_signal) {
 		g_signal_emit (GTK_OBJECT (background), signals[SETTINGS_CHANGED], 0, GDK_ACTION_COPY);
 	}
 
 	set_image_properties (background);
 	
+	g_free (filename);
+	
 	return TRUE;
 }
 
 void
 eel_background_set_image_uri (EelBackground *background, const char *image_uri)
 {
-	eel_background_set_image_uri_helper (background, image_uri, TRUE, TRUE);
-}
-
-void
-eel_background_set_image_uri_sync (EelBackground *background, const char *image_uri)
-{
-	eel_background_set_image_uri_helper (background, image_uri, TRUE, TRUE);
+	
+	
+	eel_background_set_image_uri_helper (background, image_uri, TRUE);
 }
 
 /* Use this fn to set both the image and color and avoid flash. The color isn't
@@ -597,12 +558,7 @@ static void
 eel_background_set_image_uri_and_color (EelBackground *background, GdkDragAction action,
 					const char *image_uri, const char *color)
 {
-	if (eel_strcmp (background->details->color, color) == 0 &&
-	    eel_strcmp (background->details->image_uri, image_uri) == 0) {
-		return;
-	}
-
-	eel_background_set_image_uri_helper (background, image_uri, FALSE, FALSE);
+	eel_background_set_image_uri_helper (background, image_uri, FALSE);
 	eel_background_set_color (background, color);
 
 	/* We always emit, even if the color didn't change, because the image change
@@ -634,10 +590,10 @@ eel_background_receive_dropped_background_image (EelBackground *background,
 gboolean
 eel_background_is_set (EelBackground *background)
 {
-	g_return_val_if_fail (EEL_IS_BACKGROUND (background), FALSE);
+	g_assert (EEL_IS_BACKGROUND (background));
 
 	return background->details->color != NULL
-		|| background->details->image_uri != NULL;
+		|| gnome_bg_get_filename (background->details->bg) != NULL;
 }
 
 /**
@@ -946,6 +902,15 @@ eel_background_receive_dropped_color (EelBackground *background,
 	g_free (new_gradient_spec);
 }
 
+void
+eel_background_save_to_gconf (EelBackground *background)
+{
+	GConfClient *client = gconf_client_get_default ();
+
+	if (background->details->bg)
+		gnome_bg_save_to_preferences (background->details->bg, client);
+}
+
 /* self check code */
 
 #if !defined (EEL_OMIT_SELF_CHECK)
